@@ -55,6 +55,28 @@ OSMetaClassDefineReservedUnused(IOFireWireSBP2Login, 6);
 OSMetaClassDefineReservedUnused(IOFireWireSBP2Login, 7);
 OSMetaClassDefineReservedUnused(IOFireWireSBP2Login, 8);
 
+#if   defined(__BIG_ENDIAN__)
+#define SBP2SwapHostToBigInt32Array(array, bytes)
+#define SBP2SwapBigToHostInt32Array(array, bytes)
+#elif defined(__LITTLE_ENDIAN__)
+static void SBP2SwapInt32Array(void * array, int bytes)
+{
+    if ( array && bytes )
+    {
+        UInt32 * quads = (UInt32 *) array;
+        bytes >>= 2;
+        while (bytes--)
+            quads[bytes] = OSSwapInt32(quads[bytes]);
+    }
+}
+static inline void SBP2SwapHostToBigInt32Array(void * array, int bytes)
+{ SBP2SwapInt32Array(array, bytes); }
+static inline void SBP2SwapBigToHostInt32Array(void * array, int bytes)
+{ SBP2SwapInt32Array(array, bytes); }
+#else
+#error Unknown endianess
+#endif
+
 #pragma mark -
 
 // initWithLUN
@@ -133,7 +155,7 @@ bool IOFireWireSBP2Login::initWithLUN( IOFireWireSBP2LUN * lun )
 	fUnsolicitedStatusEnableInProgress		= false;	
 	fUnsolicitedStatusEnableCommand 		= NULL;
 	
-	fSetBusyTimeoutBuffer = 0x0000000f;
+	fSetBusyTimeoutBuffer = OSSwapHostToBigInt32(0x0000000f);
 	fSetBusyTimeoutInProgress = false;
 	fSetBusyTimeoutAddress = FWAddress( 0x0000FFFF, 0xF0000210 );
 	fSetBusyTimeoutCommand = NULL;
@@ -145,6 +167,8 @@ bool IOFireWireSBP2Login::initWithLUN( IOFireWireSBP2LUN * lun )
 	fPasswordDescriptor = NULL;
 	fSuspended = false;
 	
+	fARDMAMax = 0;
+	
 	fLastORBAddress = FWAddress(0,0);
 	
 	//
@@ -198,6 +222,15 @@ bool IOFireWireSBP2Login::initWithLUN( IOFireWireSBP2LUN * lun )
 	{
 		fORBSetIterator = OSCollectionIterator::withCollection( fORBSet );
     }
+	
+	if( status == kIOReturnSuccess )
+	{		
+		OSObject * prop = fLUN->getProperty( "FWARDMAMax", gIOServicePlane );
+		if( prop != NULL )
+		{
+			fARDMAMax = ((OSNumber*)prop)->unsigned32BitValue();
+		}
+	}
 
     // the LUN that's creating us will call release() on a non-successful init
     // we'll clean up there from what ever half-allocated state we are in
@@ -293,15 +326,20 @@ IOReturn IOFireWireSBP2Login::allocateResources( void )
     // allocate and register an address space for the login ORB
     //
 
+	FWAddress host_address;
     if( status == kIOReturnSuccess )
     {
-        fLoginORBAddressSpace = IOFWSBP2PseudoAddressSpace::simpleRead( fControl, &fLoginORBAddress, sizeof(FWSBP2LoginORB), &fLoginORB );
+        fLoginORBAddressSpace = IOFWSBP2PseudoAddressSpace::simpleRead( fControl, &host_address, sizeof(FWSBP2LoginORB), &fLoginORB );
     	if ( fLoginORBAddressSpace == NULL )
         	status = kIOReturnNoMemory;
     }
 
     if( status == kIOReturnSuccess )
     {
+		fLoginORBAddress.nodeID = OSSwapHostToBigInt16( host_address.nodeID );
+		fLoginORBAddress.addressHi = OSSwapHostToBigInt16( host_address.addressHi );
+		fLoginORBAddress.addressLo = OSSwapHostToBigInt32( host_address.addressLo );
+
         status = fLoginORBAddressSpace->activate();
     }
  
@@ -329,7 +367,7 @@ IOReturn IOFireWireSBP2Login::allocateResources( void )
 
     if( status == kIOReturnSuccess )
     {
-        fReconnectORBAddressSpace = IOFWPseudoAddressSpace::simpleRead( fControl, &fReconnectORBAddress, 
+        fReconnectORBAddressSpace = IOFWPseudoAddressSpace::simpleRead( fControl, &host_address, 
 																		sizeof(FWSBP2ReconnectORB),
                                                                         &fReconnectORB );
     	if ( fReconnectORBAddressSpace == NULL )
@@ -338,6 +376,10 @@ IOReturn IOFireWireSBP2Login::allocateResources( void )
 
     if( status == kIOReturnSuccess )
     {
+        fReconnectORBAddress.nodeID = OSSwapHostToBigInt16( host_address.nodeID );
+		fReconnectORBAddress.addressHi = OSSwapHostToBigInt16( host_address.addressHi );
+		fReconnectORBAddress.addressLo = OSSwapHostToBigInt32( host_address.addressLo );
+
         status = fReconnectORBAddressSpace->activate();
     } 
 
@@ -386,7 +428,7 @@ IOReturn IOFireWireSBP2Login::allocateResources( void )
 
     if( status == kIOReturnSuccess )
     {
-        fLogoutORBAddressSpace = IOFWPseudoAddressSpace::simpleRead( fControl, &fLogoutORBAddress, sizeof(FWSBP2LogoutORB),
+        fLogoutORBAddressSpace = IOFWPseudoAddressSpace::simpleRead( fControl, &host_address, sizeof(FWSBP2LogoutORB),
                                                                      &fLogoutORB );
     	if ( fLogoutORBAddressSpace == NULL )
         	status = kIOReturnNoMemory;
@@ -394,6 +436,10 @@ IOReturn IOFireWireSBP2Login::allocateResources( void )
   
     if( status == kIOReturnSuccess )
     {
+		fLogoutORBAddress.nodeID = OSSwapHostToBigInt16( host_address.nodeID );
+		fLogoutORBAddress.addressHi = OSSwapHostToBigInt16( host_address.addressHi );
+		fLogoutORBAddress.addressLo = OSSwapHostToBigInt32( host_address.addressLo );
+
         status = fLogoutORBAddressSpace->activate();
     }
 
@@ -401,21 +447,21 @@ IOReturn IOFireWireSBP2Login::allocateResources( void )
     // prepare parts of the ORBs
     //
     
-    fLoginORB.loginResponseAddressHi = (fLoginResponseAddress.nodeID << 16) | fLoginResponseAddress.addressHi;
-    fLoginORB.loginResponseAddressLo = fLoginResponseAddress.addressLo;
-    fLoginORB.loginResponseLength = sizeof( FWSBP2LoginResponse );
-    fLoginORB.lun = fLUN->getLUNumber();
-    FWKLOG( ("IOFireWireSBP2Login<0x%08lx>::allocateResources lun number = %d\n", (UInt32)this, fLoginORB.lun) );
-    fLoginORB.statusFIFOAddressHi = (fStatusBlockAddress.nodeID << 16) | fStatusBlockAddress.addressHi;
-    fLoginORB.statusFIFOAddressLo = fStatusBlockAddress.addressLo;
+    fLoginORB.loginResponseAddressHi = OSSwapHostToBigInt32((fLoginResponseAddress.nodeID << 16) | fLoginResponseAddress.addressHi);
+    fLoginORB.loginResponseAddressLo = OSSwapHostToBigInt32(fLoginResponseAddress.addressLo);
+    fLoginORB.loginResponseLength = OSSwapHostToBigInt16(sizeof( FWSBP2LoginResponse ));
+    fLoginORB.lun = OSSwapHostToBigInt16(fLUN->getLUNumber());
+    FWKLOG( ("IOFireWireSBP2Login<0x%08lx>::allocateResources lun number = %d\n", (UInt32)this, OSSwapBigToHostInt16(fLoginORB.lun)) );
+    fLoginORB.statusFIFOAddressHi = OSSwapHostToBigInt32((fStatusBlockAddress.nodeID << 16) | fStatusBlockAddress.addressHi);
+    fLoginORB.statusFIFOAddressLo = OSSwapHostToBigInt32(fStatusBlockAddress.addressLo);
 
-    fReconnectORB.options = 3 |  0x8000;   // reconnect | notify
-    fReconnectORB.statusFIFOAddressHi = (fReconnectStatusBlockAddress.nodeID << 16) | fReconnectStatusBlockAddress.addressHi;
-    fReconnectORB.statusFIFOAddressLo = fReconnectStatusBlockAddress.addressLo;
+    fReconnectORB.options = OSSwapHostToBigInt16(3 |  0x8000);   // reconnect | notify
+    fReconnectORB.statusFIFOAddressHi = OSSwapHostToBigInt32((fReconnectStatusBlockAddress.nodeID << 16) | fReconnectStatusBlockAddress.addressHi);
+    fReconnectORB.statusFIFOAddressLo = OSSwapHostToBigInt32(fReconnectStatusBlockAddress.addressLo);
 
-    fLogoutORB.options = 7 | 0x8000;  	// logout | notify
-    fLogoutORB.statusFIFOAddressHi = (fStatusBlockAddress.nodeID << 16) | fStatusBlockAddress.addressHi;
-    fLogoutORB.statusFIFOAddressLo = fStatusBlockAddress.addressLo;
+    fLogoutORB.options = OSSwapHostToBigInt16(7 | 0x8000);  	// logout | notify
+    fLogoutORB.statusFIFOAddressHi = OSSwapHostToBigInt32((fStatusBlockAddress.nodeID << 16) | fStatusBlockAddress.addressHi);
+    fLogoutORB.statusFIFOAddressLo = OSSwapHostToBigInt32(fStatusBlockAddress.addressLo);
     
 	//
     // create command for writing the management agent
@@ -1134,9 +1180,9 @@ IOReturn IOFireWireSBP2Login::setPassword( IOMemoryDescriptor * memory )
 	
 			if( status == kIOReturnSuccess )
 			{
-				fLoginORB.passwordLength = len;
-				fLoginORB.password[0] = 0x0000ffff & fPasswordAddress.addressHi;
-				fLoginORB.password[1] = fPasswordAddress.addressLo;
+				fLoginORB.passwordLength = OSSwapHostToBigInt16(len);
+				fLoginORB.password[0] = OSSwapHostToBigInt32(0x0000ffff & fPasswordAddress.addressHi);
+				fLoginORB.password[1] = OSSwapHostToBigInt32(fPasswordAddress.addressLo);
 			}
 			
 		}
@@ -1206,9 +1252,9 @@ IOReturn IOFireWireSBP2Login::setPassword( void * buf, UInt32 len )
         {
             fPasswordBuf = buf;
             fPasswordLen = len;
-            fLoginORB.passwordLength = len;
-            fLoginORB.password[0] = 0x0000ffff & fPasswordAddress.addressHi;
-            fLoginORB.password[1] = fPasswordAddress.addressLo;
+            fLoginORB.passwordLength = OSSwapHostToBigInt16(len);
+            fLoginORB.password[0] = OSSwapHostToBigInt32(0x0000ffff & fPasswordAddress.addressHi);
+            fLoginORB.password[1] = OSSwapHostToBigInt32(fPasswordAddress.addressLo);
         }
         
     }
@@ -1314,10 +1360,10 @@ IOReturn IOFireWireSBP2Login::executeLogin( void )
     // set options
     FWKLOG(( "IOFireWireSBP2Login<0x%08lx> : fLoginFlags : 0x%08lx\n", (UInt32)this, fLoginFlags ));
 
-    fLoginORB.options = 0x0000 | 0x8000;		// login | notify
-    fLoginORB.options |= (fReconnectTime << 4);
+    fLoginORB.options = OSSwapHostToBigInt16(0x0000 | 0x8000);		// login | notify
+    fLoginORB.options |= OSSwapHostToBigInt16(fReconnectTime << 4);
     if( fLoginFlags & kFWSBP2ExclusiveLogin )
-        fLoginORB.options |= 0x1000;
+        fLoginORB.options |= OSSwapHostToBigInt16(0x1000);
 
     // set to correct generation
     fLoginWriteCommand->reinit( FWAddress(0x0000ffff, 0xf0000000 + (fManagementOffset << 2)),
@@ -1524,11 +1570,14 @@ UInt32 IOFireWireSBP2Login::statusBlockWrite( UInt16 nodeID, IOFWSpeed &speed, F
 {
     FWKLOG( ( "IOFireWireSBP2Login<0x%08lx> : status block write\n", (UInt32)this ) );
 
+    
 	if( len < 8 )
 	{
 	//	IOLog( "IOFireWireSBP2Login<0x%08lx> : illegal status block length = %d\n", this, len );
 		return kFWResponseComplete;
 	}
+
+    bzero( &fStatusBlock, sizeof(fStatusBlock) );
 	
     //
     if( len < sizeof(fStatusBlock) )
@@ -1536,6 +1585,24 @@ UInt32 IOFireWireSBP2Login::statusBlockWrite( UInt16 nodeID, IOFWSpeed &speed, F
     else
         bcopy( buf, &fStatusBlock, sizeof(fStatusBlock));
 
+	const void * client_status_block = &fStatusBlock;
+
+#if 0
+	// compatibility hack for Craig's layer
+	
+	FWSBP2StatusBlock host_status_block;
+	
+	bzero( &host_status_block, sizeof(host_status_block) );
+	bcopy( &fStatusBlock, &host_status_block, sizeof(host_status_block));
+
+    host_status_block.orbOffsetHi = OSSwapBigToHostInt16(fStatusBlock.orbOffsetHi);
+    host_status_block.orbOffsetLo = OSSwapBigToHostInt32(fStatusBlock.orbOffsetLo);
+	
+    SBP2SwapBigToHostInt32Array( &host_status_block.status, sizeof(host_status_block.status) );
+	
+	client_status_block = &host_status_block;	
+#endif
+
     switch( fLoginState )
     {
         case kLoginStateLoggingIn:
@@ -1554,19 +1621,22 @@ UInt32 IOFireWireSBP2Login::statusBlockWrite( UInt16 nodeID, IOFWSpeed &speed, F
             {
                 FWKLOG( ( "IOFireWireSBP2Login<0x%08lx> : successful login\n", (UInt32)this ) );
   
-                // get login ID and fetch agent address
-                fLoginID = fLoginResponse.loginID;
-                fReconnectORB.loginID = fLoginID;  // set id for reconnect;
-            
+				// get login ID and fetch agent address
+                fLoginID = OSSwapBigToHostInt16(fLoginResponse.loginID);
+                fReconnectORB.loginID = OSSwapHostToBigInt16(fLoginID);  // set id for reconnect;
+
                 // set reconnect_hold, some devices indicate it
-                if( fLoginResponse.length >= 16 )
-                    fReconnectHold = (fLoginResponse.reconnectHold & 0x7fff) + 1;
+                if( OSSwapBigToHostInt16(fLoginResponse.length) >= 16 )
+                    fReconnectHold = (OSSwapBigToHostInt16(fLoginResponse.reconnectHold) & 0x7fff) + 1;
                 else
                     fReconnectHold = 1;
 
+				UInt32 commandBlockAgentAddressHi = OSSwapBigToHostInt32(fLoginResponse.commandBlockAgentAddressHi);
+				UInt32 commandBlockAgentAddressLo = OSSwapBigToHostInt32(fLoginResponse.commandBlockAgentAddressLo);
+				
 				// set fetch agent reset address
-				fFetchAgentResetAddress = FWAddress( fLoginResponse.commandBlockAgentAddressHi & 0x0000ffff,
-													 fLoginResponse.commandBlockAgentAddressLo + 0x00000004 );
+				fFetchAgentResetAddress = FWAddress( commandBlockAgentAddressHi & 0x0000ffff,
+													 commandBlockAgentAddressLo + 0x00000004 );
 				
 				fFetchAgentResetCommand->reinit( fFetchAgentResetAddress,
 												&fFetchAgentResetBuffer, 1,
@@ -1577,13 +1647,13 @@ UInt32 IOFireWireSBP2Login::statusBlockWrite( UInt16 nodeID, IOFWSpeed &speed, F
                 // set fetch agent address
 				if( fFastStartSupported )
 				{
-					fFetchAgentAddress = FWAddress( fLoginResponse.commandBlockAgentAddressHi & 0x0000ffff,
-													fLoginResponse.commandBlockAgentAddressLo + (fFastStartOffset << 2));
+					fFetchAgentAddress = FWAddress( commandBlockAgentAddressHi & 0x0000ffff,
+													commandBlockAgentAddressLo + (fFastStartOffset << 2));
 				}
 				else
 				{
-					fFetchAgentAddress = FWAddress( fLoginResponse.commandBlockAgentAddressHi & 0x0000ffff,
-													fLoginResponse.commandBlockAgentAddressLo + 0x00000008);
+					fFetchAgentAddress = FWAddress( commandBlockAgentAddressHi & 0x0000ffff,
+													commandBlockAgentAddressLo + 0x00000008);
 				}
 				
                 fFetchAgentWriteCommand->reinit( fFetchAgentAddress,  // tis determined
@@ -1593,8 +1663,8 @@ UInt32 IOFireWireSBP2Login::statusBlockWrite( UInt16 nodeID, IOFWSpeed &speed, F
 				fFetchAgentWriteCommand->setRetries( 0 );
 				
 				// set doorbell reset address
-				fDoorbellAddress = FWAddress( fLoginResponse.commandBlockAgentAddressHi & 0x0000ffff,
-											  fLoginResponse.commandBlockAgentAddressLo + 0x00000010 );
+				fDoorbellAddress = FWAddress( commandBlockAgentAddressHi & 0x0000ffff,
+											  commandBlockAgentAddressLo + 0x00000010 );
 				
 				fDoorbellCommand->reinit( fDoorbellAddress,
 										  &fDoorbellBuffer, 1,
@@ -1603,8 +1673,8 @@ UInt32 IOFireWireSBP2Login::statusBlockWrite( UInt16 nodeID, IOFWSpeed &speed, F
 				fDoorbellCommand->updateNodeID( fLoginGeneration, fLoginNodeID );
 		
 				// set unsolicited status enable address
-				fUnsolicitedStatusEnableAddress = FWAddress( fLoginResponse.commandBlockAgentAddressHi & 0x0000ffff,
-															 fLoginResponse.commandBlockAgentAddressLo + 0x00000014 );
+				fUnsolicitedStatusEnableAddress = FWAddress( commandBlockAgentAddressHi & 0x0000ffff,
+															 commandBlockAgentAddressLo + 0x00000014 );
 				
 				fUnsolicitedStatusEnableCommand->reinit( fUnsolicitedStatusEnableAddress,
 														 &fUnsolicitedStatusEnableBuffer, 1,
@@ -1614,28 +1684,28 @@ UInt32 IOFireWireSBP2Login::statusBlockWrite( UInt16 nodeID, IOFWSpeed &speed, F
 		
 				fLoginState = kLoginStateConnected;
 				
-                completeLogin( kIOReturnSuccess, buf, len, &fLoginResponse );
+                completeLogin( kIOReturnSuccess, client_status_block, len, &fLoginResponse );
             }
             else
             {
                 FWKLOG( ( "IOFireWireSBP2Login<0x%08lx> : login failed\n", (UInt32)this ) );
                 fLoginState = kLoginStateIdle;
 
-                completeLogin( kIOReturnError, buf, len, NULL );
+                completeLogin( kIOReturnError, client_status_block, len, NULL );
             }
             break;
 
         case kLoginStateConnected:
             // orb related or unsolicited
             
-            if( ((*((UInt8*)buf)) & 0xc0) == 0x80 )
+            if( (fStatusBlock.details & 0xc0) == 0x80 )
             {
                 // send unsolicited status
                 if( fUnsolicitedStatusNotifyCallback != NULL )
                 {
                     FWSBP2NotifyParams		params;
 
-                    params.message = buf;
+                    params.message = client_status_block;
                     params.length = len;
                     params.notificationEvent = kFWSBP2UnsolicitedStatus;
                     params.generation = fLoginGeneration;
@@ -1649,7 +1719,12 @@ UInt32 IOFireWireSBP2Login::statusBlockWrite( UInt16 nodeID, IOFWSpeed &speed, F
 
                 // find ORB
                 bool found = false;
-                FWAddress notificationAddress = *((FWAddress*)buf);
+				
+                FWAddress notificationAddress;
+				notificationAddress.nodeID = 0x0000;
+				notificationAddress.addressHi = OSSwapBigToHostInt16(fStatusBlock.orbOffsetHi);
+				notificationAddress.addressLo = OSSwapBigToHostInt32(fStatusBlock.orbOffsetLo);
+
                 IOFireWireSBP2ORB * item = NULL;
                 fORBSetIterator->reset();
                 while( (found != true) && (item = (IOFireWireSBP2ORB *) fORBSetIterator->getNextObject())  )
@@ -1665,8 +1740,7 @@ UInt32 IOFireWireSBP2Login::statusBlockWrite( UInt16 nodeID, IOFWSpeed &speed, F
                 
                 FWKLOG( ( "IOFireWireSBP2Login<0x%08lx> : solicited found = %d\n", (UInt32)this, found ) );
 				
-				FWSBP2StatusBlock * statusBlock = (FWSBP2StatusBlock*)buf;
-				bool deadBitIsSet = statusBlock->details & 0x08;
+				bool deadBitIsSet = fStatusBlock.details & 0x08;
                 
                 if( found )
                 {
@@ -1697,7 +1771,7 @@ UInt32 IOFireWireSBP2Login::statusBlockWrite( UInt16 nodeID, IOFWSpeed &speed, F
                     {
                         FWSBP2NotifyParams		params;
      
-                        params.message = buf;
+                        params.message = client_status_block;
                         params.length = len;
                         params.notificationEvent = kFWSBP2NormalCommandStatus;
                         params.generation = fLoginGeneration;
@@ -1732,14 +1806,14 @@ UInt32 IOFireWireSBP2Login::statusBlockWrite( UInt16 nodeID, IOFWSpeed &speed, F
 				fUnsolicitedStatusEnableRequested = false;
 				
                 // get login ID and fetch agent address
-                fLoginID = fLoginResponse.loginID;
-                completeLogout( kIOReturnSuccess, buf, len );
+                fLoginID = OSSwapBigToHostInt16(fLoginResponse.loginID);
+                completeLogout( kIOReturnSuccess, client_status_block, len );
             }
             else
             {
                 FWKLOG( ( "IOFireWireSBP2Login<0x%08lx> : logout failed!?\n", (UInt32)this ) );
                 fLoginState = kLoginStateIdle;
-				completeLogout( kIOReturnError, buf, len );
+				completeLogout( kIOReturnError, client_status_block, len );
             }
 			
 			// all tasks are aborted
@@ -1819,7 +1893,8 @@ void IOFireWireSBP2Login::resumeNotify( void )
     FWKLOG( ( "IOFireWireSBP2Login<0x%08lx> : resume notify\n", (UInt32)this ) );
 
     UInt32 generation = fControl->getGeneration();
-
+	fPhysicalAccessEnabled = fUnit->isPhysicalAccessEnabled();
+	
 	fSuspended = false;
 	
 	executeSetBusyTimeout(); //can handle interruption
@@ -2155,6 +2230,8 @@ UInt32 IOFireWireSBP2Login::reconnectStatusBlockWrite( UInt16 nodeID, IOFWSpeed
         return kFWResponseComplete;
     }
 
+    bzero( &fReconnectStatusBlock, sizeof(fReconnectStatusBlock) );
+
     if( len < sizeof(fReconnectStatusBlock) )
         bcopy( buf, &fReconnectStatusBlock, len);
     else
@@ -2358,7 +2435,7 @@ IOReturn IOFireWireSBP2Login::executeLogout( void )
 	if( fLoginState == kLoginStateConnected )
 	{
 		fLoginState = kLoginStateLoggingOut;
-		fLogoutORB.loginID = fLoginID;
+		fLogoutORB.loginID = OSSwapHostToBigInt16(fLoginID);
 	
 		// set to correct generation
 		fLogoutWriteCommand->reinit( FWAddress(0x0000ffff, 0xf0000000 + (fManagementOffset << 2)),
@@ -2676,6 +2753,7 @@ IOReturn IOFireWireSBP2Login::executeORB( IOFireWireSBP2ORB * orb )
     {
         // retries failed fetch agent writes up to four times
         orb->setFetchAgentWriteRetries( 20 );
+		orb->setFetchAgentWriteRetryInterval( 1000 );
         prepareORBForExecution(orb);
     }
 
@@ -2724,12 +2802,12 @@ IOReturn IOFireWireSBP2Login::executeORB( IOFireWireSBP2ORB * orb )
 		
 			if( orbAddress.addressHi == 0 && orbAddress.addressLo == 0 )
 			{
-				orbAddress.nodeID = 0x8000;
+				orbAddress.nodeID = OSSwapHostToBigInt16(0x8000);
 			}
 			else
 			{
-				orbAddress.nodeID = fLocalNodeID;
-			}		
+				orbAddress.nodeID = OSSwapHostToBigInt16(fLocalNodeID);
+			}
 			descriptor->writeBytes( 0, &orbAddress, sizeof(FWAddress) );
 		}
 
@@ -2739,7 +2817,11 @@ IOReturn IOFireWireSBP2Login::executeORB( IOFireWireSBP2ORB * orb )
 		{
 			FWAddress orbAddress(0,0);
 			orb->getORBAddress( &orbAddress );
-			orbAddress.nodeID = fLocalNodeID;
+			
+			orbAddress.nodeID = OSSwapHostToBigInt16(fLocalNodeID);
+			orbAddress.addressHi = OSSwapHostToBigInt16(orbAddress.addressHi);
+			orbAddress.addressLo = OSSwapHostToBigInt32(orbAddress.addressLo);
+
 			descriptor->writeBytes( 8, &orbAddress, sizeof(FWAddress) );
 		}
 		
@@ -2760,7 +2842,12 @@ IOReturn IOFireWireSBP2Login::executeORB( IOFireWireSBP2ORB * orb )
     {
         if( commandFlags & kFWSBP2CommandImmediate )
         {
-			orb->getORBAddress( &fLastORBAddress );
+			FWAddress orbAddress(0,0);
+			orb->getORBAddress( &orbAddress );
+			
+			fLastORBAddress.nodeID = OSSwapHostToBigInt16(orbAddress.nodeID);
+			fLastORBAddress.addressHi = OSSwapHostToBigInt16(orbAddress.addressHi);
+			fLastORBAddress.addressLo = OSSwapHostToBigInt32(orbAddress.addressLo);
 			fLastORB = orb;
 
 			if( fFetchAgentWriteCommandInUse )
@@ -2898,10 +2985,13 @@ void IOFireWireSBP2Login::fetchAgentWriteComplete( IOReturn status, IOFireWireNu
 				// retry
 				retries--;
 				fLastORB->setFetchAgentWriteRetries( retries );
-
-				// family does not delay on these acks and rcodes
 				
-				startFetchAgentRetryTimer( kFetchAgentRetryInterval );
+				// arithmetic increase in retry time
+				UInt32 interval = fLastORB->getFetchAgentWriteRetryInterval();
+				fLastORB->setFetchAgentWriteRetryInterval( interval + 1000 );
+ 
+				// family does not delay on these acks and rcodes
+				startFetchAgentRetryTimer( interval );
 			
 				return;
 			}
@@ -3229,7 +3319,7 @@ void IOFireWireSBP2Login::unsolicitedStatusEnableComplete( IOReturn status, IOFi
 
 void IOFireWireSBP2Login::setBusyTimeoutRegisterValue( UInt32 timeout )
 {
-	fSetBusyTimeoutBuffer = timeout;
+	fSetBusyTimeoutBuffer = OSSwapHostToBigInt32(timeout);
 	executeSetBusyTimeout();
 }
 
@@ -3240,7 +3330,7 @@ IOReturn IOFireWireSBP2Login::executeSetBusyTimeout( void )
 			
 	fSetBusyTimeoutInProgress = true;
 	fSetBusyTimeoutCommand->reinit( fSetBusyTimeoutAddress,
-									&fSetBusyTimeoutBuffer, 1,
+								    &fSetBusyTimeoutBuffer, 1,
 								    IOFireWireSBP2Login::setBusyTimeoutCompleteStatic, 
 								    this, true );
 
@@ -3291,8 +3381,13 @@ IOReturn IOFireWireSBP2Login::appendORB( IOFireWireSBP2ORB * orb )
     IOReturn status = kIOReturnSuccess;
     if( fLastORB != NULL && fLastORB != orb )
     {
-        orb->getORBAddress( &fLastORBAddress );
-        setNextORBAddress( fLastORB, fLastORBAddress );
+		FWAddress orb_address(0,0);
+        orb->getORBAddress( &orb_address );
+        setNextORBAddress( fLastORB, orb_address );
+		
+		fLastORBAddress.nodeID = OSSwapHostToBigInt16(orb_address.nodeID);
+		fLastORBAddress.addressHi = OSSwapHostToBigInt16(orb_address.addressHi);
+		fLastORBAddress.addressLo = OSSwapHostToBigInt32(orb_address.addressLo);
         fLastORB = orb;
     }
     else
@@ -3344,7 +3439,7 @@ void IOFireWireSBP2Login::setAddressLoForLoginORBAndResponse( UInt32 addressLoOR
 	space = OSDynamicCast( IOFWSBP2PseudoAddressSpace, fLoginORBAddressSpace );
 	if( space != NULL )
 	{
-		fLoginORBAddress.addressLo = addressLoORBMasked;
+		fLoginORBAddress.addressLo = OSSwapHostToBigInt32(addressLoORBMasked);
 		space->setAddressLo( addressLoORBMasked );
 	}
 	
@@ -3352,13 +3447,31 @@ void IOFireWireSBP2Login::setAddressLoForLoginORBAndResponse( UInt32 addressLoOR
 	if( space != NULL )
 	{
 		fLoginResponseAddress.addressLo = addressLoResponseMasked;
-		fLoginORB.loginResponseAddressLo = fLoginResponseAddress.addressLo;
+		fLoginORB.loginResponseAddressLo = OSSwapHostToBigInt32(fLoginResponseAddress.addressLo);
 		space->setAddressLo( addressLoResponseMasked );
 	}
 	
 	fControl->openGate();
 }
 
+// isPhysicalAccessEnabled
+//
+//
+
+bool IOFireWireSBP2Login::isPhysicalAccessEnabled( void )
+{
+	return fPhysicalAccessEnabled;
+}
+
+// getARDMMax
+//
+//
+
+UInt32 IOFireWireSBP2Login::getARDMMax( void )
+{
+	return fARDMAMax;
+}
+
 #pragma mark -
 //////////////////////////////////////////////////////////////////////////////////////////
 // friend class wrappers
