@@ -135,8 +135,11 @@ protected:
 protected:
    
     // reserved for future use
-    struct ExpansionData { };
-    ExpansionData *reserved; 
+    struct ExpansionData 
+	{
+		bool fInCriticalSection; 
+	};
+    ExpansionData * fExpansionData; 
          
     IOFireWireSBP2LUN * 		fLUN;
     IOFireWireUnit *			fUnit;
@@ -179,6 +182,8 @@ protected:
 	
 	IOMemoryMap *				fResponseMap;
 	
+	bool						fCompleting;
+	
 	////////////////////////////////////////        
 
     // init / destroy
@@ -319,11 +324,13 @@ public:
 
 protected:
 
-   virtual void setORBToDummy( IOFireWireSBP2ORB * orb );
+	virtual void setORBToDummy( IOFireWireSBP2ORB * orb );
+
+	virtual void suspendedNotify();
 
 private:
 
-    OSMetaClassDeclareReservedUnused(IOFireWireSBP2ManagementORB, 0);
+    OSMetaClassDeclareReservedUsed(IOFireWireSBP2ManagementORB, 0);
     OSMetaClassDeclareReservedUnused(IOFireWireSBP2ManagementORB, 1);
     OSMetaClassDeclareReservedUnused(IOFireWireSBP2ManagementORB, 2);
     OSMetaClassDeclareReservedUnused(IOFireWireSBP2ManagementORB, 3);
