@@ -29,6 +29,8 @@
 #include <IOKit/firewire/IOFireWireController.h>
 #undef FIREWIREPRIVATE
 
+#include <IOKit/firewire/IOFWSimpleContiguousPhysicalAddressSpace.h>
+
 OSDefineMetaClassAndStructors( IOFireWireSBP2ORB, IOCommand );
 
 OSMetaClassDefineReservedUnused(IOFireWireSBP2ORB, 0);
@@ -62,7 +64,8 @@ bool IOFireWireSBP2ORB::initWithLogin( IOFireWireSBP2Login * login )
     fMaxPayloadSize					= 0;
     fCommandFlags					= 0;
     fTimeoutDuration				= 0;
-
+	fDMACommand						= NULL;
+	fConstraintOptions				= 0;
     // init super
    if( !IOCommand::init() )
         return false;
@@ -82,6 +85,9 @@ IOReturn IOFireWireSBP2ORB::allocateResources( void )
 {
      IOReturn status = kIOReturnSuccess;
 
+	status = setBufferConstraints( kFWSBP2MaxPageClusterSize, 1, 0 );
+	
+//	IOLog("IOFireWireSBP2ORB::allocateResources - setBufferConstraints, status = 0x%08lx\n", status );
     //
     // create ORB
     //
@@ -93,6 +99,8 @@ IOReturn IOFireWireSBP2ORB::allocateResources( void )
 		status = allocateORB( orbSize );
 	}
 
+//	IOLog("IOFireWireSBP2ORB::allocateResources - allocateORB, status = 0x%08lx\n", status );
+
 	//
 	// create page table
 	//
@@ -102,6 +110,8 @@ IOReturn IOFireWireSBP2ORB::allocateResources( void )
         status = allocatePageTable( PAGE_SIZE / sizeof(FWSBP2PTE) );  // default size
 	}
 
+//	IOLog("IOFireWireSBP2ORB::allocateResources - allocatePageTable, status = 0x%08lx\n", status );
+
 	//
 	// create timer
 	//
@@ -121,6 +131,8 @@ IOReturn IOFireWireSBP2ORB::allocateResources( void )
         deallocateORB();
         deallocatePageTable();
     }
+
+//	IOLog("IOFireWireSBP2ORB::allocateResources - status = 0x%08lx\n", status );
      
     return status;
 }
@@ -141,7 +153,7 @@ void IOFireWireSBP2ORB::free( void )
     deallocatePageTable();
     deallocateBufferAddressSpace();
 	deallocateORB();
-	
+		
     IOCommand::free();
 }
 
@@ -156,77 +168,55 @@ void IOFireWireSBP2ORB::free( void )
 IOReturn IOFireWireSBP2ORB::allocateORB( UInt32 orbSize )
 {
 	IOReturn	status = kIOReturnSuccess;
-		
-	//
-    // allocate ORB
-    //
-    
-    // allocate mem for page table
-	fORBDescriptor = IOBufferMemoryDescriptor::withOptions( kIODirectionOutIn | kIOMemoryUnshared, orbSize, orbSize );
-    if( fORBDescriptor == NULL )
-        status =  kIOReturnNoMemory;
-		
-    if( status == kIOReturnSuccess )
-    {
-        fORBDescriptor->setLength( orbSize );
 
-        IOPhysicalLength lengthOfSegment = 0;
-        IOPhysicalAddress phys = fORBDescriptor->getPhysicalSegment( 0, &lengthOfSegment );
-        FWKLOG( ( "IOFireWireSBP2ORB<0x%08lx> : ORB segment length = %d\n", (UInt32)this, lengthOfSegment ) );
-            
-        if( lengthOfSegment != 0 )
-        {
-            fORBPhysicalAddress = phys;
-            FWKLOG( ( "IOFireWireSBP2ORB<0x%08lx> : ORB - phys = 0x%08lx\n", (UInt32)this, fORBPhysicalAddress ) );
-        }
-        else
-        {
-            status =  kIOReturnNoMemory;
-        }
-    }
-	
+	IOFWSimpleContiguousPhysicalAddressSpace * physical_space;
 	if( status == kIOReturnSuccess )
-	{
-		// if physically contiguous, must be virtually contiguous
-		fORBBuffer = (FWSBP2ORB*)fORBDescriptor->getBytesNoCopy();  
-
-		// clear orb
-		bzero( fORBBuffer, orbSize );
-	}
-
-    // allocate and register a physical address space for the ORB
-
-    if( status == kIOReturnSuccess )
     {
-        fORBPhysicalAddressSpace = fUnit->createPhysicalAddressSpace( fORBDescriptor );
-       	if ( fORBPhysicalAddressSpace == NULL )
+        physical_space = fUnit->createSimpleContiguousPhysicalAddressSpace( orbSize, kIODirectionOut );
+     	if( physical_space == NULL )
         	status = kIOReturnNoMemory;
     }
 
-    if( status == kIOReturnSuccess )
+//	IOLog( "IOFireWireSBP2ORB::allocateORB - 1 status = 0x%08lx\n", status );
+
+	if( status == kIOReturnSuccess )
+	{
+		fORBPhysicalAddressSpace = (IOFWAddressSpace*)physical_space;
+		fORBDescriptor = physical_space->getMemoryDescriptor();
+		fORBPhysicalAddress = physical_space->getFWAddress();
+		fORBBuffer = (FWSBP2ORB *)physical_space->getVirtualAddress();
+	}
+
+//	IOLog( "IOFireWireSBP2ORB::allocateORB - 2 status = 0x%08lx\n", status );
+	
+	if( status == kIOReturnSuccess )
     {
         status = fORBPhysicalAddressSpace->activate();
     }
 
-    // allocate and register a pseudo address space for the ORB
-	
+//	IOLog( "IOFireWireSBP2ORB::allocateORB - 3 status = 0x%08lx\n", status );
+
     if( status == kIOReturnSuccess )
     {
 		//zzz shouldn't be able to write to ORBs
-		//zzz of course when run physically there's nothing to stop writes anyway
+		//zzz of course when running via the physical unit there's nothing to stop writes anyway
 		
         fORBPseudoAddressSpace = IOFWPseudoAddressSpace::simpleRW( fControl, &fORBPseudoAddress, fORBDescriptor );
        	if ( fORBPseudoAddressSpace == NULL )
         	status = kIOReturnNoMemory;
     }
 
+//	IOLog( "IOFireWireSBP2ORB::allocateORB - 4 status = 0x%08lx\n", status );
+
     if( status == kIOReturnSuccess )
     {
-		fORBPseudoAddress.addressHi &= 0xffff;	// Mask off nodeID part.
         status = fORBPseudoAddressSpace->activate();
 		
-		FWKLOG( ( "IOFireWireSBP2ORB<0x%08lx> : created orb at phys: 0x%04lx.%08lx psuedo: 0x%04lx.%08lx of size %d\n", (UInt32)this, 0, fORBPhysicalAddress, fORBPseudoAddress.addressHi, fORBPseudoAddress.addressLo, orbSize ) );
+		FWKLOG( ( "IOFireWireSBP2ORB<0x%08lx> : created orb at phys: 0x%04lx.%08lx psuedo: 0x%04lx.%08lx of size %d\n", 
+				(UInt32)this, fORBPhysicalAddress.addressHi, fORBPhysicalAddress.addressLo, fORBPseudoAddress.addressHi, fORBPseudoAddress.addressLo, orbSize ) );
     }
+
+//	IOLog( "IOFireWireSBP2ORB::allocateORB - 5 status = 0x%08lx\n", status );
 	
 	if( status != kIOReturnSuccess )
 	{
@@ -255,14 +245,6 @@ void IOFireWireSBP2ORB::deallocateORB( void )
         fORBPseudoAddressSpace->release();
 		fORBPseudoAddressSpace = NULL;
 	}
-
-    // free mem
-    if( fORBDescriptor != NULL )
-	{
-		fORBBuffer = NULL;
-        fORBDescriptor->release();
-		fORBDescriptor = NULL;
-	}
 }
 
 /////////////////////////////////////////////////////////////////////
@@ -277,38 +259,38 @@ void IOFireWireSBP2ORB::prepareORBForExecution( void )
 {
     
     // make sure orb points nowhere
-    fORBBuffer->nextORBAddressHi = 0x80000000;
-    fORBBuffer->nextORBAddressLo = 0x00000000;
+    fORBBuffer->nextORBAddressHi = OSSwapHostToBigInt32(0x80000000);
+    fORBBuffer->nextORBAddressLo = OSSwapHostToBigInt32(0x00000000);
 
     // update data descriptor with local node ID
     UInt32 generation;	// Hmm, shouldn't this be checked?
     UInt16 unitNode;
     UInt16 localNode;
     fUnit->getNodeIDGeneration( generation, unitNode, localNode );
-    fORBBuffer->dataDescriptorHi &= 0x0000ffff;
-    fORBBuffer->dataDescriptorHi |= ((UInt32)localNode) << 16;
+    fORBBuffer->dataDescriptorHi &= OSSwapHostToBigInt32(0x0000ffff);
+    fORBBuffer->dataDescriptorHi |= OSSwapHostToBigInt32(((UInt32)localNode) << 16);
 
     // clear options
-    fORBBuffer->options &= 0x000f;
+    fORBBuffer->options &= OSSwapHostToBigInt16(0x000f);
 
     // mark for notify if requested
     if( fCommandFlags & kFWSBP2CommandCompleteNotify )
-        fORBBuffer->options |= 0x8000;
+        fORBBuffer->options |= OSSwapHostToBigInt16(0x8000);
 
     // mark ORB rq_fmt. if kFWSBP2CommandNormalORB is set, the zero is already in place.
     // the driver is not supposed to set more than one of these flags
     if( fCommandFlags & kFWSBP2CommandReservedORB )
-        fORBBuffer->options |= 0x2000;
+        fORBBuffer->options |= OSSwapHostToBigInt16(0x2000);
 
     if( fCommandFlags & kFWSBP2CommandVendorORB )
-        fORBBuffer->options |= 0x4000;
+        fORBBuffer->options |= OSSwapHostToBigInt16(0x4000);
 
     if( fCommandFlags & kFWSBP2CommandDummyORB )
-        fORBBuffer->options |= 0x6000;
+        fORBBuffer->options |= OSSwapHostToBigInt16(0x6000);
 
     // mark as "read" if requested
     if( fCommandFlags & kFWSBP2CommandTransferDataFromTarget )
-        fORBBuffer->options |= 0x0800;
+        fORBBuffer->options |= OSSwapHostToBigInt16(0x0800);
 
     //
     // set speed
@@ -318,15 +300,15 @@ void IOFireWireSBP2ORB::prepareORBForExecution( void )
     switch( speed )
     {
 		case kFWSpeed800MBit:
-			fORBBuffer->options |= 0x0300;
+			fORBBuffer->options |= OSSwapHostToBigInt16(0x0300);
 			break;
 
         case kFWSpeed400MBit:
-            fORBBuffer->options |= 0x0200;
+            fORBBuffer->options |= OSSwapHostToBigInt16(0x0200);
              break;
 
         case kFWSpeed200MBit:
-            fORBBuffer->options |= 0x0100;
+            fORBBuffer->options |= OSSwapHostToBigInt16(0x0100);
             break;
 
         default:
@@ -339,7 +321,15 @@ void IOFireWireSBP2ORB::prepareORBForExecution( void )
     //
 
     UInt32 transferSizeBytes = 4096; // start at max packet size
-
+	
+	// clip by ARMDMAMax for performance
+	UInt32 ARDMAMax = fLogin->getARDMMax();
+	bool physical_unit_on = fLogin->isPhysicalAccessEnabled();
+	if( !physical_unit_on && (ARDMAMax != 0) )
+	{
+		transferSizeBytes = ARDMAMax;
+	}
+	
     // trim by max payload sizes
     UInt32 loginMaxPayloadSize = fLogin->getMaxPayloadSize();
     if( loginMaxPayloadSize != 0 && loginMaxPayloadSize < transferSizeBytes )
@@ -363,7 +353,7 @@ void IOFireWireSBP2ORB::prepareORBForExecution( void )
         transferSizeLog = maxPackLog;
 
     // set transfer size, actual max is 2 ^ (size + 2) bytes (or 2 ^ size quads)
-    fORBBuffer->options |= transferSizeLog << 4;
+    fORBBuffer->options |= OSSwapHostToBigInt16(transferSizeLog << 4);
 }
 
 // prepareFastStartPacket
@@ -424,6 +414,148 @@ void IOFireWireSBP2ORB::prepareFastStartPacket( IOBufferMemoryDescriptor * descr
 	descriptor->setLength( offset );
 }
 
+#if 0
+IOReturn checkMemoryInRange( IOMemoryDescriptor * memory, UInt64 mask, IODMACommand * dma_command_arg )
+{
+	IOReturn status = kIOReturnSuccess;
+
+	if( memory == NULL )
+	{
+		status = kIOReturnBadArgument;
+	}
+	
+	//
+	// setup
+	//
+	
+	bool memory_prepared = false;
+	if( dma_command_arg == NULL )
+	{
+		if( status == kIOReturnSuccess )
+		{
+			status = memory->prepare( kIODirectionInOut );
+		}
+		
+		if( status == kIOReturnSuccess )
+		{
+			memory_prepared = true;
+		}
+	}
+	
+		
+	UInt64 length = 0;
+	
+	if( status == kIOReturnSuccess )
+	{
+		length = memory->getLength();
+	}
+	
+	IODMACommand * dma_command = dma_command_arg;
+	bool dma_command_prepared = false;
+	if( dma_command == NULL )
+	{
+		if( status == kIOReturnSuccess )
+		{
+			dma_command = IODMACommand::withSpecification( 
+													kIODMACommandOutputHost64,		// segment function
+													64,								// max address bits
+													0,								// max segment size
+													(IODMACommand::MappingOptions)(IODMACommand::kMapped | IODMACommand::kIterateOnly),		// IO mapped & don't bounce buffer
+													0,							// max transfer size
+													0,								// page alignment
+													NULL,							// mapper
+													NULL );							// refcon
+			if( dma_command == NULL )
+				status = kIOReturnError;
+			
+		}
+		
+		if( status == kIOReturnSuccess )
+		{
+			// set memory descriptor and don't prepare it
+			status = dma_command->setMemoryDescriptor( memory, false ); 
+		}	
+
+		if( status == kIOReturnSuccess )
+		{
+			status = dma_command->prepare( 0, length, true );
+		}
+
+		if( status == kIOReturnSuccess )
+		{
+			dma_command_prepared = true;
+		}
+	}
+	else
+	{
+#if 0
+	dma_command->setMemoryDescriptor( memory, false ); 
+	dma_command->prepare( 0, length, true );
+	dma_command_prepared = true;
+#endif	
+	}
+	//
+	// check ranges
+	//
+
+	if( status == kIOReturnSuccess )
+	{
+		IOLog( "checkSegments - length = %d\n", length  );
+	
+		UInt64 offset = 0;
+		while( (offset < length) && (status == kIOReturnSuccess) )
+		{
+			IODMACommand::Segment64 segments[10];
+			UInt32 num_segments = 10;
+			status = dma_command->gen64IOVMSegments( &offset, segments, &num_segments );
+			if( status == kIOReturnSuccess )
+			{
+				for( UInt32 i = 0; i < num_segments; i++ )
+				{
+					IOLog( "checkSegments - offset = 0x%016llx segments[%d].fIOVMAddr = 0x%016llx, fLength = %d\n", offset, i, segments[i].fIOVMAddr, segments[i].fLength  );
+						
+					if( (segments[i].fIOVMAddr & (~mask)) )
+					{
+						IOLog( "checkSegmentsFailed - 0x%016llx & 0x%016llx\n", segments[i].fIOVMAddr, mask );
+						status = kIOReturnNotPermitted;
+					//	break;
+					}
+				}
+			}
+			else
+			{
+				IOLog( "checkSegments - offset = %lld 0x%08lx\n", offset, status  );
+			}
+		}
+	}
+	
+	//
+	// clean up
+	//
+	
+	if( dma_command_prepared )
+	{
+		dma_command->complete();
+		dma_command_prepared = false;
+	}
+		
+	if( dma_command && (dma_command_arg == NULL) )
+	{
+		dma_command->release();
+		dma_command = NULL;
+	}
+	
+	if( memory_prepared )
+	{
+		memory->complete();
+		memory_prepared = false;
+	}
+	
+	return status;
+}
+
+#endif
+
 //////////////////////////////////////////////////////////////////////////////////////////
 // timeouts
 //
@@ -555,7 +687,7 @@ IOReturn IOFireWireSBP2ORB::allocatePageTable( UInt32 entryCount )
 {
 	IOReturn	status = kIOReturnSuccess;
 	
-	deallocatePageTable();
+	deallocatePageTable();	
 	
 	//
     // create page table
@@ -563,29 +695,93 @@ IOReturn IOFireWireSBP2ORB::allocatePageTable( UInt32 entryCount )
     
     // allocate mem for page table
     fPageTableSize = sizeof(FWSBP2PTE) * entryCount;
-    fPageTableDescriptor = IOBufferMemoryDescriptor::withOptions( kIODirectionOutIn | kIOMemoryUnshared, fPageTableSize, PAGE_SIZE );
+	UInt64 phys_mask = fControl->getFireWirePhysicalAddressMask();
+	fPageTableDescriptor = IOBufferMemoryDescriptor::inTaskWithPhysicalMask( kernel_task, kIODirectionOutIn, fPageTableSize, phys_mask );
     if( fPageTableDescriptor == NULL )
         status =  kIOReturnNoMemory;
 
     if( status == kIOReturnSuccess )
     {
         fPageTableDescriptor->setLength( fPageTableSize );
+	}
 
-        IOPhysicalLength lengthOfSegment = 0;
-        IOPhysicalAddress phys = fPageTableDescriptor->getPhysicalSegment( 0, &lengthOfSegment );
-    //    FWKLOG( ( "IOFireWireSBP2ORB<0x%08lx> : pageTable segment length = %d\n", (UInt32)this, lengthOfSegment ) );
-            
-        if( lengthOfSegment != 0 )
-        {
-            fPageTablePhysicalAddress = phys;
- //           FWKLOG( ( "IOFireWireSBP2ORB<0x%08lx> : pageTable - phys = 0x%08lx\n", (UInt32)this, fPageTablePhysicalAddress ) );
-        }
-        else
-        {
-            status =  kIOReturnNoMemory;
-        }
-    }
+	bool dma_command_prepared = false;
+	IODMACommand * dma_command = NULL;
+	if( status == kIOReturnSuccess )
+    {
+		dma_command = IODMACommand::withSpecification( 
+												kIODMACommandOutputHost64,		// segment function
+												48,								// max address bits
+												fPageTableSize,					// max segment size
+												(IODMACommand::MappingOptions)(IODMACommand::kMapped | IODMACommand::kIterateOnly),		// IO mapped & don't bounce buffer
+												fPageTableSize,					// max transfer size
+												0,								// no alignment
+												NULL,							// mapper
+												NULL );							// refcon
+		if( dma_command == NULL )
+			status = kIOReturnNoMemory;
+	}
 
+//	IOLog( "IOFireWireSBP2ORB::allocatePageTable (1) - status = 0x%08lx\n", status );
+	
+	if( status == kIOReturnSuccess )
+	{
+		// set memory descriptor and don't prepare it
+		status = dma_command->setMemoryDescriptor( fPageTableDescriptor, false ); 
+	}	
+
+	if( status == kIOReturnSuccess )
+	{
+		status = dma_command->prepare( 0, fPageTableSize, true );
+	}
+
+	if( status == kIOReturnSuccess )
+	{
+		dma_command_prepared = true;
+	}
+
+//	IOLog( "IOFireWireSBP2ORB::allocatePageTable (2) - status = 0x%08lx\n", status );
+	
+	IODMACommand::Segment64 segment;
+	UInt32 numSegments = 1;
+	if( status == kIOReturnSuccess )
+	{
+		UInt64 offset = 0;
+		status = dma_command->gen64IOVMSegments( &offset, &segment, &numSegments );
+	}
+	
+	if( status == kIOReturnSuccess )
+	{
+		if( numSegments != 1 )
+		{
+			status = kIOReturnNoMemory;
+		}
+	}
+
+//	IOLog( "IOFireWireSBP2ORB::allocatePageTable (3) - status = 0x%08lx\n", status );
+	
+	if( status == kIOReturnSuccess )
+	{
+		fPageTablePhysicalLength = segment.fLength;
+		fPageTablePhysicalAddress.nodeID = 0x0000;		// invalid node id
+		fPageTablePhysicalAddress.addressHi = (segment.fIOVMAddr >> 32) & 0x000000000000ffffULL;
+		fPageTablePhysicalAddress.addressLo = segment.fIOVMAddr & 0x00000000ffffffffULL;
+	}
+
+	if( dma_command_prepared )
+	{
+		dma_command->complete();
+		dma_command_prepared = false;
+	}
+	
+	if( dma_command )
+	{
+		dma_command->release();
+		dma_command = NULL;
+	}
+
+//	IOLog( "IOFireWireSBP2ORB::allocatePageTable (1) - status = 0x%08lx segment = 0x%016llx len = %d\n", status, segment.fIOVMAddr, segment.fLength );
+	
     // allocate and register a physical address space for the page table
 
     if( status == kIOReturnSuccess )
@@ -659,6 +855,136 @@ void IOFireWireSBP2ORB::deallocatePageTable( void )
 	fPageTableSize = 0;
 }
 
+// setBufferConstraints
+//
+//
+
+IOReturn IOFireWireSBP2ORB::setBufferConstraints( UInt64 maxSegmentSize, UInt32 alignment, UInt32 options )
+{
+	IOReturn status = kIOReturnSuccess;
+
+	if( fBufferAddressSpaceAllocated )
+	{
+		status = kIOReturnBusy;
+	}
+	
+	if( status == kIOReturnSuccess )
+	{
+		fConstraintOptions = options;
+		
+		deallocateBufferAddressSpace();
+		
+		//
+		// create DMACommand
+		//
+		
+		if( status == kIOReturnSuccess )
+		{
+			UInt32 address_bits = fControl->getFireWirePhysicalAddressBits();
+#if 0
+		// testing
+		//	address_bits = 29;
+#endif
+			
+			UInt64 segment_size = maxSegmentSize;
+			if( (segment_size > kFWSBP2MaxPageClusterSize) || (segment_size == 0) )
+				segment_size = kFWSBP2MaxPageClusterSize;
+					
+			fDMACommand = IODMACommand::withSpecification( 
+													kIODMACommandOutputHost64,		// segment function
+													address_bits,					// max address bits
+													segment_size,					// max segment size
+													IODMACommand::kMapped,			// IO mapped & allow bounce buffering
+													0,								// max transfer size
+													alignment,						// alignment
+													NULL,							// mapper
+													NULL );							// refcon
+			if( fDMACommand == NULL )
+				status = kIOReturnError;
+		}
+	}
+
+	// allocate and register an address space for the buffers
+	
+//	IOLog( "IOFireWireSBP2ORB::setBufferConstraints - (1) status = 0x%08lx\n", status );
+	
+	if( status == kIOReturnSuccess )
+	{
+		fBufferAddressSpace = fUnit->createPhysicalAddressSpace( NULL );
+		if( fBufferAddressSpace == NULL )
+		{
+			status = kIOReturnNoMemory;
+		}
+	}
+
+//	IOLog( "IOFireWireSBP2ORB::setBufferConstraints - (2) status = 0x%08lx\n", status );
+
+	if( status == kIOReturnSuccess )
+	{
+		((IOFWPhysicalAddressSpace*)fBufferAddressSpace)->setDMACommand( fDMACommand );
+	}
+	
+	return status;
+}
+
+// prepareBufferAddressSpace
+//
+//
+
+IOReturn IOFireWireSBP2ORB::prepareBufferAddressSpace( IOMemoryDescriptor * memoryDescriptor )
+{
+	if( fBufferAddressSpaceAllocated )
+	{
+		completeBufferAddressSpace();
+	}
+
+	fBufferDescriptor = memoryDescriptor;
+	fBufferDescriptor->retain();
+
+	((IOFWPhysicalAddressSpace*)fBufferAddressSpace)->setMemoryDescriptor( fBufferDescriptor );
+	fBufferAddressSpace->activate();
+	
+	if( fConstraintOptions & kFWSBP2ConstraintForceDoubleBuffer )
+	{
+		((IOFWPhysicalAddressSpace*)fBufferAddressSpace)->synchronize( IODMACommand::kForceDoubleBuffer | kIODirectionOut );
+	}
+	
+	fBufferAddressSpaceAllocated = true;
+	
+	return kIOReturnSuccess;
+}
+
+// completeBufferAddressSpace
+//
+//
+
+IOReturn IOFireWireSBP2ORB::completeBufferAddressSpace( void )
+{
+	if( fBufferAddressSpace )
+	{
+		fBufferAddressSpace->deactivate();
+		((IOFWPhysicalAddressSpace*)fBufferAddressSpace)->setMemoryDescriptor( NULL );
+	}
+	
+	if( fBufferDescriptor )
+	{
+		fBufferDescriptor->release();
+		fBufferDescriptor = NULL;
+	}
+	
+	fBufferAddressSpaceAllocated = false;
+
+	if( fORBBuffer )
+	{
+		fORBBuffer->dataDescriptorHi = 0;
+		fORBBuffer->dataDescriptorLo = 0;
+		fORBBuffer->options &= OSSwapHostToBigInt16(0xfff0); // no page table
+		fORBBuffer->dataSize = 0;
+	}
+	
+	return kIOReturnSuccess;
+}
+
 // deallocateBufferAddressSpace
 //
 //
@@ -671,23 +997,33 @@ void IOFireWireSBP2ORB::deallocateBufferAddressSpace( void )
 	
     if( fBufferAddressSpaceAllocated )
     {
-        fBufferAddressSpace->deactivate();
-        fBufferAddressSpace->release();
-        fBufferDescriptor->release();
-        fBufferAddressSpaceAllocated = false;
-    }
+		completeBufferAddressSpace();		
+	}
+	
+	if( fBufferAddressSpace )
+	{
+		fBufferAddressSpace->release();
+		fBufferAddressSpace = NULL;
+	}
+	
+	if( fDMACommand )
+	{
+		fDMACommand->release();
+		fDMACommand = NULL;
+	}
 
 	// no page table
 	if( fORBBuffer )
 	{
 		fORBBuffer->dataDescriptorHi	= 0;
 		fORBBuffer->dataDescriptorLo	= 0;
-		fORBBuffer->options				&= 0xfff0;	// no page table
+		fORBBuffer->options				&= OSSwapHostToBigInt16(0xfff0);
+	// no page table
 		fORBBuffer->dataSize			= 0;
 	}
 	
 	fPTECount = 0;
-	    
+
     fControl->openGate();
 }
 
@@ -749,15 +1085,15 @@ IOReturn IOFireWireSBP2ORB::setCommandBuffers( IOMemoryDescriptor * memoryDescri
     IOReturn status = kIOReturnSuccess;
  
 	fControl->closeGate();
-	
+
     //
-    // deallocate old physical address space
+    // deallocate previous mapping
     //
 
     // I obsolete the old address space even if we fail at mapping a new one
     // that seems like it would be the more expected behavior
-    
-    deallocateBufferAddressSpace();
+
+	completeBufferAddressSpace();
 
 	//
 	// bail if no memory descriptor
@@ -767,11 +1103,37 @@ IOReturn IOFireWireSBP2ORB::setCommandBuffers( IOMemoryDescriptor * memoryDescri
     {
 		fControl->openGate();
 
+	//	IOLog( "IOFireWireSBP2ORB::setCommandBuffers status = 0x%08lx\n", status );
+	
 		return kIOReturnSuccess;
 	}
-   
+
+#if 0   
    	// memoryDescriptor is valid from here on
-   	
+
+	IOReturn check_status = checkMemoryInRange( memoryDescriptor, 0x000000001fffffff, NULL );
+#endif
+
+	//
+	// map buffers
+	//
+	
+	// allocate and register an address space for the buffers
+	
+	if( status == kIOReturnSuccess )
+	{
+		status = prepareBufferAddressSpace( memoryDescriptor );
+	}
+
+#if 0
+	if( check_status != kIOReturnSuccess )
+	{
+		IOLog( "After prepare\n" );
+		check_status = checkMemoryInRange( memoryDescriptor, 0x000000001fffffff, fDMACommand );
+		IOLog( "After prepare = 0x%08lx\n", check_status );
+	}
+#endif
+	
    	//
    	// fix up length if necessary
    	//
@@ -782,13 +1144,13 @@ IOReturn IOFireWireSBP2ORB::setCommandBuffers( IOMemoryDescriptor * memoryDescri
 
         if( length == 0 )
         {
-            length = memoryDescriptor->getLength();
+            length = fDMACommand->getMemoryDescriptor()->getLength();
 		}
 		
 #if FWLOGGING
         // I occasionally get memory descriptors with bogus lengths
 
-        UInt32 tempLength = memoryDescriptor->getLength();
+        UInt32 tempLength = fDMACommand->getMemoryDescriptor->getLength();
 
 		if(length != tempLength)
 		{
@@ -798,7 +1160,7 @@ IOReturn IOFireWireSBP2ORB::setCommandBuffers( IOMemoryDescriptor * memoryDescri
 #endif
         
 	}
-        
+               
     //
     // write page table
     //
@@ -825,12 +1187,34 @@ IOReturn IOFireWireSBP2ORB::setCommandBuffers( IOMemoryDescriptor * memoryDescri
 			
 			while( pos < (length + offset) )
 			{
-	    		IOPhysicalAddress 	phys;
-   				IOPhysicalLength 	lengthOfSegment;
+	    		UInt64 	phys;
+   				UInt64 	lengthOfSegment;
 
 				// get next segment
+	
+				IODMACommand::Segment64 segment;
+				UInt32 numSegments = 1;
+				if( status == kIOReturnSuccess )
+				{
+					UInt64 dma_pos = pos;  // don't mangle pos
+					status = fDMACommand->gen64IOVMSegments( &dma_pos, &segment, &numSegments );
+				//	IOLog( "IOFireWireSBP2ORB::setCommandBuffers - pos = %d, status = 0x%08lx\n", pos, status );
+				//	IOSleep( 10 );
+				}
 				
-				phys = memoryDescriptor->getPhysicalSegment( pos, &lengthOfSegment );
+				if( status == kIOReturnSuccess )
+				{
+					if( numSegments != 1 )
+					{
+						status = kIOReturnNoMemory;
+					}
+				}
+
+				if( status == kIOReturnSuccess )
+				{
+					phys = segment.fIOVMAddr;
+					lengthOfSegment = segment.fLength;
+				}
 	
 				// nothing more to map
 				
@@ -842,6 +1226,8 @@ IOReturn IOFireWireSBP2ORB::setCommandBuffers( IOMemoryDescriptor * memoryDescri
 				}
 				
 				// map until we are done or we run out of segment
+
+				// DMA command constraints should make this code unnecessary
 				
 				while( (pos < (length + offset)) && (lengthOfSegment != 0) )
 				{
@@ -849,7 +1235,7 @@ IOReturn IOFireWireSBP2ORB::setCommandBuffers( IOMemoryDescriptor * memoryDescri
     				UInt32		toMap = 0;
 
 					// 64k max page table entry, so we do it in chunks
-					
+
 					if( lengthOfSegment > kFWSBP2MaxPageClusterSize )
 						step = kFWSBP2MaxPageClusterSize;
 					else
@@ -874,18 +1260,20 @@ IOReturn IOFireWireSBP2ORB::setCommandBuffers( IOMemoryDescriptor * memoryDescri
 						{
 							FWSBP2PTE entry;
 		
-							entry.segmentLength = toMap;
-							entry.segmentBaseAddressHi = 0x0000;
-							entry.segmentBaseAddressLo = phys;
+							entry.segmentLength = OSSwapHostToBigInt16( toMap );
+							entry.segmentBaseAddressHi = OSSwapHostToBigInt16( ((phys >> 32) & 0x000000000000ffffULL) );
+							entry.segmentBaseAddressLo = OSSwapHostToBigInt32( (phys & 0x00000000ffffffffULL) );
 		
 							fPageTableDescriptor->writeBytes( (pte-1) * sizeof(FWSBP2PTE), &entry, sizeof(FWSBP2PTE) );
+					
+						//	IOLog( "IOFireWireSBP2ORB<0x%08lx> : PTE = %d, size = %d\n", (UInt32)this, pte-1, toMap  );
 							
 		 //                  FWKLOG( ( "IOFireWireSBP2ORB<0x%08lx> : PTE = %d, size = %d\n", (UInt32)this, pte-1, toMap  ) );
-		
-							// move to new page table entry and beginning of unmapped memory
-							phys += toMap;
-							pos += toMap;
-						}		
+						}
+						
+						// move to new page table entry and beginning of unmapped memory
+						phys += toMap;
+						pos += toMap;
 					}
 				}
 			}
@@ -916,34 +1304,7 @@ IOReturn IOFireWireSBP2ORB::setCommandBuffers( IOMemoryDescriptor * memoryDescri
 			}			
 		}
 	}
-	
-	//
-	// map buffers
-	//
-	
-	// allocate and register an address space for the buffers
-	
-	if( status == kIOReturnSuccess )
-	{
-		fBufferDescriptor = memoryDescriptor;
-		fBufferDescriptor->retain();
-		fBufferAddressSpace = fUnit->createPhysicalAddressSpace( memoryDescriptor );
-		if( fBufferAddressSpace == NULL )
-		{
-			status = kIOReturnNoMemory;
-		}
-	}
-
-	if( status == kIOReturnSuccess )
-	{
-		status = fBufferAddressSpace->activate();
-	}
-
-	if( status == kIOReturnSuccess )
-	{
-		fBufferAddressSpaceAllocated = true;
-	}
-    
+	    
     //
     // fill in orb
     //
@@ -955,7 +1316,7 @@ IOReturn IOFireWireSBP2ORB::setCommandBuffers( IOMemoryDescriptor * memoryDescri
 			// no page table
             fORBBuffer->dataDescriptorHi	= 0;
             fORBBuffer->dataDescriptorLo	= 0;
-            fORBBuffer->options				&= 0xfff0;	// no page table
+            fORBBuffer->options				&= OSSwapHostToBigInt16(0xfff0);	// no page table
             fORBBuffer->dataSize			= 0;
 			fPTECount = 0;
         }
@@ -968,39 +1329,46 @@ IOReturn IOFireWireSBP2ORB::setCommandBuffers( IOMemoryDescriptor * memoryDescri
 			// very strictly speaking, if we don't use a page table, the buffer
 			// must be quadlet-aligned.  SBP-2 is vague about this.  We'll enforce it.
 
-			if( pte == 1 && !(firstEntry.segmentBaseAddressLo & 0x00000003) )  // make sure quadlet aligned
+			if( pte == 1 && !(firstEntry.segmentBaseAddressLo & OSSwapHostToBigInt32(0x00000003)) )  // make sure quadlet aligned
 			{
 				// directly addressed buffer
 				fORBBuffer->dataDescriptorHi	= 0;			// tbd later
 				fORBBuffer->dataDescriptorLo	= firstEntry.segmentBaseAddressLo;
-				fORBBuffer->options				&= 0xfff0;		// no page table
+				fORBBuffer->options				&= OSSwapHostToBigInt16(0xfff0);		// no page table
 				fORBBuffer->dataSize			= firstEntry.segmentLength;
 				fPTECount = 0;
 			}
-			else if( pte * sizeof(FWSBP2PTE) < PAGE_SIZE )
+			else if( pte * sizeof(FWSBP2PTE) <= fPageTablePhysicalLength )
 			{
 				// use physical unit
-				fORBBuffer->dataDescriptorHi	= 0;			// tbd later
-				fORBBuffer->dataDescriptorLo	= fPageTablePhysicalAddress;
-				fORBBuffer->options				&= 0xfff0;
-				fORBBuffer->options				|= 0x0008;		// unrestricted page table
-				fORBBuffer->dataSize			= pte;
+				fORBBuffer->dataDescriptorHi	= OSSwapHostToBigInt32(fPageTablePhysicalAddress.addressHi);			// tbd later
+				fORBBuffer->dataDescriptorLo	= OSSwapHostToBigInt32(fPageTablePhysicalAddress.addressLo);
+				fORBBuffer->options				&= OSSwapHostToBigInt16(0xfff0);
+				fORBBuffer->options				|= OSSwapHostToBigInt16(0x0008);		// unrestricted page table
+				fORBBuffer->dataSize			= OSSwapHostToBigInt16(pte);
 				fPTECount = pte;
 			}
 			else
 			{
 				// use software address space
-				fORBBuffer->dataDescriptorHi	= fPageTablePseudoAddress.addressHi;			// tbd later
-				fORBBuffer->dataDescriptorLo	= fPageTablePseudoAddress.addressLo;
-				fORBBuffer->options				&= 0xfff0;
-				fORBBuffer->options				|= 0x0008;		// unrestricted page table
-				fORBBuffer->dataSize			= pte;			
+				fORBBuffer->dataDescriptorHi	= OSSwapHostToBigInt32(fPageTablePseudoAddress.addressHi);			// tbd later
+				fORBBuffer->dataDescriptorLo	= OSSwapHostToBigInt32(fPageTablePseudoAddress.addressLo);
+				fORBBuffer->options				&= OSSwapHostToBigInt16(0xfff0);
+				fORBBuffer->options				|= OSSwapHostToBigInt16(0x0008);		// unrestricted page table
+				fORBBuffer->dataSize			= OSSwapHostToBigInt16(pte);			
 				fPTECount = pte;
 			}
 		}
     }
- 
+
+	if( status != kIOReturnSuccess )
+	{
+		completeBufferAddressSpace();
+	}
+	
     fControl->openGate();
+
+//	IOLog( "IOFireWireSBP2ORB::setCommandBuffers return status = 0x%08lx\n", status );
        
     return status;
 }
@@ -1049,13 +1417,13 @@ IOReturn IOFireWireSBP2ORB::setCommandBlock( IOMemoryDescriptor * memory )
     
     UInt32 maxCommandBlockSize = fLogin->getMaxCommandBlockSize();
     IOByteCount length = memory->getLength();
-    
+	
     if( length > maxCommandBlockSize )
         return kIOReturnNoMemory;
 
     bzero( &fORBBuffer->commandBlock[0], maxCommandBlockSize );
     memory->readBytes(0, &fORBBuffer->commandBlock[0], length );
-
+	
     fControl->openGate();
 
     return kIOReturnSuccess;
@@ -1138,19 +1506,20 @@ UInt32 IOFireWireSBP2ORB::getCommandGeneration( void )
 void IOFireWireSBP2ORB::getORBAddress( FWAddress * address )
 {
 	if( fCommandFlags & kFWSBP2CommandVirtualORBs )
+	{
 		*address = fORBPseudoAddress;
+	}
 	else
 	{
-		address->addressHi = 0L;
-		address->addressLo = fORBPhysicalAddress;
+		*address = fORBPhysicalAddress;
 	}
 }
 
 // sets the address of the next ORB field
 void IOFireWireSBP2ORB::setNextORBAddress( FWAddress address )
 {
-    fORBBuffer->nextORBAddressHi = address.addressHi;
-    fORBBuffer->nextORBAddressLo = address.addressLo;
+    fORBBuffer->nextORBAddressHi = OSSwapHostToBigInt32((UInt32)address.addressHi);
+    fORBBuffer->nextORBAddressLo = OSSwapHostToBigInt32(address.addressLo);
 }
 
 // get login
@@ -1162,7 +1531,7 @@ IOFireWireSBP2Login * IOFireWireSBP2ORB::getLogin( void )
 void IOFireWireSBP2ORB::setToDummy( void )
 {
     // set rq_fmt to 3 (NOP) in case it wasn't fetched yet
-    fORBBuffer->options |= 0x6000;
+    fORBBuffer->options |= OSSwapHostToBigInt16(0x6000);
 }
 
 bool IOFireWireSBP2ORB::isAppended( void )
@@ -1185,6 +1554,24 @@ void IOFireWireSBP2ORB::setFetchAgentWriteRetries( UInt32 retries )
     fFetchAgentWriteRetries = retries;
 }
 
+// getFetchAgentWriteRetryInterval
+//
+//
+
+UInt32 IOFireWireSBP2ORB::getFetchAgentWriteRetryInterval( void )
+{
+	return fFetchAgentWriteRetryInterval;
+}
+
+// setFetchAgentWriteRetryInterval
+//
+//
+
+void IOFireWireSBP2ORB::setFetchAgentWriteRetryInterval( UInt32 interval )
+{
+	fFetchAgentWriteRetryInterval = interval;
+}
+
 //////////////////////////////////////////////////////////////////////////////////////////
 // friend class wrappers
 
