@@ -2,24 +2,21 @@
  * Copyright (c) 1998-2000 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ *
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
+ *
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
- * 
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
+ *
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -35,6 +32,12 @@
 
 #include <IOKit/IOBufferMemoryDescriptor.h>
 #include <IOKit/IOUserClient.h>
+#include <IOKit/IODMACommand.h>
+
+enum
+{
+    kFWSBP2ConstraintForceDoubleBuffer		= (1 << 0)
+};
 
 // login option flags
 enum
@@ -160,27 +163,27 @@ protected:
     IOFireWireSBP2LUN * 	fLUN;
     IOFireWireUnit *		fUnit;
     IOFireWireController *	fControl;
-    void *					fUnused1;
-    void *	  				fUnused2;
-
+    IODMACommand *			fDMACommand;
+	void *					fUnused2;
+	
     UInt32			fCommandFlags;
     UInt32			fMaxPayloadSize;
     UInt32			fTimeoutDuration;
     UInt32			fGeneration;
     void *			fRefCon;
-    
+
 	//
     // orb
     //
 	
-	IOBufferMemoryDescriptor *	fORBDescriptor;
+	IOMemoryDescriptor *		fORBDescriptor;
     FWSBP2ORB *					fORBBuffer;
 	
     FWAddress 					fORBPseudoAddress;
     IOFWAddressSpace *			fORBPseudoAddressSpace;
 
     IOFWAddressSpace *			fORBPhysicalAddressSpace;
-	IOPhysicalAddress			fORBPhysicalAddress;
+	FWAddress					fORBPhysicalAddress;
 
 	//
     // page table
@@ -190,7 +193,8 @@ protected:
     IOBufferMemoryDescriptor *	fPageTableDescriptor;
 
     IOFWAddressSpace *			fPageTablePhysicalAddressSpace;
-	IOPhysicalAddress			fPageTablePhysicalAddress;
+	FWAddress					fPageTablePhysicalAddress;
+	UInt32						fPageTablePhysicalLength;
 	
 	IOFWAddressSpace *			fPageTablePseudoAddressSpace;   
 	FWAddress					fPageTablePseudoAddress;
@@ -215,6 +219,9 @@ protected:
 	
     UInt32					fFetchAgentWriteRetries;
     UInt32					fPTECount;
+    UInt32					fFetchAgentWriteRetryInterval;
+
+	UInt32					fConstraintOptions;
 	
     virtual IOReturn allocateResources( void );
     virtual void free( void );
@@ -358,7 +365,7 @@ public:
 
     /*! 
         @function getCommandFlags
-        @abstract Gets configuration flags for the ORB.
+        @abstract Sets configuration flags for the ORB.
         @discussion Returns the current configuration flags set on this ORB.
         @result Return The current ORB flags.
     */
@@ -468,6 +475,25 @@ protected:
 
     virtual void prepareFastStartPacket( IOBufferMemoryDescriptor * descriptor );
 
+	UInt32 getFetchAgentWriteRetryInterval( void );
+    void setFetchAgentWriteRetryInterval( UInt32 interval );
+
+	IOReturn completeBufferAddressSpace( void );
+	IOReturn prepareBufferAddressSpace( IOMemoryDescriptor * memoryDescriptor );
+
+public:
+
+    /*!
+		@function setBufferConstraints
+		@abstract Configures page table generation parameters
+		@discussion Sets the maximums size of any page table segment and the required alignemnt.  Double buffering
+		may be used to satisfy these constraints. The only supported option is kFWSBP2ConstraintForceDoubleBuffer which
+		forces a page aligned double buffering of the entire descriptor.
+        @result May return an error if there is a problem allocating the underlying resources or if buffers are currently attached.
+	*/
+
+	IOReturn setBufferConstraints( UInt64 maxSegmentSize, UInt32 alignment, UInt32 options = 0);
+
 };
     
 #endif
\ No newline at end of file
